#version 330 core

#define POSITION_RECONSTRUCTION

uniform sampler2D normalTex;
uniform sampler2D colorTex;
uniform sampler2D emissionTex;
uniform sampler2D rgmaTex;
#ifndef POSITION_RECONSTRUCTION
uniform sampler2D positionTex;
#endif
uniform sampler2D depthTex;

in vec4 texCoord0;

out vec4 fragColor;

float linearizeDepth(float d);
vec4 getViewSpacePosition(vec2 texCoord, float linearDepth);
vec3 lightFragment(vec4 position, vec3 normal, vec4 color, vec3 emission, vec4 rgma, float fragDepth);

void main()
{
#ifdef POSITION_RECONSTRUCTION
	float rawDepth = texture(depthTex, texCoord0.xy).r;
	float depth = linearizeDepth(rawDepth);
	vec4 position   = getViewSpacePosition(texCoord0.xy, depth);
#else
	vec4 position = texture(positionTex, texCoord0.xy);
#endif
	
	vec4 color = texture(colorTex, texCoord0.xy);
    vec3 emission = texture(emissionTex, texCoord0.xy).rgb;
    vec3 normal = texture(normalTex, texCoord0.xy).rgb;
    vec4 rgma = texture(rgmaTex, texCoord0.xy);
    
    fragColor = vec4(lightFragment(position, normal, color, emission, rgma, rawDepth), color.a);
}