#version 330 core

#define POSITION_RECONSTRUCTION

uniform sampler2DArray colorTex;
uniform sampler2DArray emissionTex;
uniform sampler2DArray rgmaTex;
uniform sampler2DArray normalTex;

uniform vec4 emission;
uniform vec4 rgma;

in vec3 normal;
in vec3 tangent;
in vec3 bitangent;
in vec4 texCoord0;
in vec4 color;
#ifndef POSITION_RECONSTRUCTION
in vec4 position;
#endif

layout(location = 0) out vec4 fragNorm;
layout(location = 1) out vec4 fragColor;
layout(location = 2) out vec4 fragEmission;
layout(location = 3) out vec4 fragRGMA;
layout(location = 4) out vec4 fragPosition;

void main()
{
	vec3 norm = 2.0 * (texture(normalTex, texCoord0.xyz).rgb - 0.5);
	fragNorm = vec4(norm.x * tangent + norm.y * bitangent + norm.z * normal, 1.0);
	fragColor = vec4(texture(colorTex, texCoord0.xyz).rgb * color.rgb, 1.0);
	fragEmission = texture(emissionTex, texCoord0.xyz) * emission;
	fragRGMA = texture(rgmaTex, texCoord0.xyz) * rgma;
	#ifndef POSITION_RECONSTRUCTION
	fragPosition = position;
	#endif
}