#version 330 core

uniform sampler1D biasTex;

layout(triangles) in;
layout(triangle_strip, max_vertices = VERTEX_COUNT) out;

in vec4 texCoord0[];
in vec3 normal[];

layout(std140) uniform ShadowMaps {
	int numShadowMaps;
	mat4 shadowMapMatrices[SHADOW_MAP_DEPTH];
};

out vec3 smTexCoord0;

void main()
{
	float constantBias = texelFetch(biasTex, 0, 0).r;
	for(int l = 0; l<SHADOW_MAP_DEPTH; l++)
	{
		if(l >= numShadowMaps)
			break;
	    gl_Layer = l;
	    vec2 bias = texelFetch(biasTex, l, 0).xy;
		vec3 zRow = normalize(vec3(shadowMapMatrices[l][0][2], shadowMapMatrices[l][1][2], shadowMapMatrices[l][2][2]));
	    
	    for (int i = 0; i < 3; ++i)
	    {
		    //vec4 normal = normalize(shadowMapMatrices[l] * vec4(normal[i], 0.0));
		    //float normalZ = normal.z;
		    
	        float normalZ = dot(normal[i], zRow);
	        vec4 position = shadowMapMatrices[l] * gl_in[i].gl_Position;
	        //Note: Constants prevent bias going to 0/infinity when orthogonal/parallel to light
	        gl_Position = position + vec4(0.0, 0.0, bias.x + bias.y * sqrt(1.00-normalZ*normalZ)/(0.1 + abs(normalZ)), 0.0);
	        smTexCoord0 = texCoord0[i].xyz;
	        EmitVertex();
	    }
	    EndPrimitive();
	}
}  