;Competent Audio comes with a biquad filter built in (in native code).
;That functionality is exposed to DSPs via interrupts [0x60-0x6A] (see ca_dsp_interrupts).
;
;This program runs a biquad lowpass filter on the audio data.
;The cutoff and resonance parameters can be changed by passing 1-2 floats as user arguments.

jmp .initializer

$cutoff     dword       440.0
$resonance  dword       4.4
$channels   dword       0
$rate       dword       0
$filters    dword[32]   0

.initializer:
    lcons t4, 4         ;constant 4 bytes
    add sp, sp, t4      ;skip version number
    pop t0              ;sample rate
    stor $rate, t0
    pop t1              ;channel count
    stor $channels, t1
    
    lcons r0, .initMP
    lcons r1, .mainMP
    lcons r2, .main
    halt

.initMP:
    mov t0, sp
    int 0
    push t0
    call .msgProc
    int 1
    
    load r1, $rate
    load r2, $channels
    lcons r3, $filters  ;filter array pointer
    lcons t0, 0         ;iteration index
    load t1, $cutoff
    load t2, $resonance
    lcons t4, 4         ;constant 4 bytes
    
    .newFilterStart:
        jae t0, r2, .newFilterEnd
        
        int 0           ;enter
        push r1         ;arg 0 - sample rate
        int 0x60        ;create filter
        stor_p r3, r0   ;save handle
        int 1           ;exit
        
        int 0           ;enter
        push t2         ;arg 2 - resonance
        push t1         ;arg 1 - cutoff
        push r0         ;arg 0 - filter handle
        int 0x63        ;configure lowpass
        int 1           ;exit
        
        inc t0
        add r3, r3, t4
        jmp .newFilterStart
    .newFilterEnd:
    
    lcons r0, 0
    halt

.mainMP:
    mov t0, sp
    int 0
    push t0
    call .msgProc
    int 1
    
    load r2, $channels
    lcons r3, $filters  ;filter array pointer
    lcons t0, 0          ;iteration index
    load t1, $cutoff
    load t2, $resonance
    lcons t4, 4         ;constant 4 bytes
    
    .adjustFilterStart:
        jae t0, r2, .adjustFilterEnd
        load_p r0, r3      ;filter handle
        
        int 0              ;enter
        push t2            ;arg 2 - resonance
        push t1            ;arg 1 - cutoff
        push r0            ;arg 0 - filter handle
        int 0x63           ;configure lowpass
        int 1              ;exit
        
        inc t0
        add r3, r3, t4
        jmp .adjustFilterStart
    .adjustFilterEnd:
    
    lcons r0, 0
    halt

.msgProc:
    lcons t4, 4
    pop t3          ;pointer to message
    
    load_p t0, t3   ;user arg count
    add t3, t3, t4
    
    jz t0, .msgProcEnd
        ;Has cutoff arg
        load_p t1, t3   ;cutoff arg
        add t3, t3, t4
        stor $cutoff, t1
        dec t0          ;decrement arg count
    jz t0, .msgProcEnd
        ;Has resonance arg
        load_p t2, t3   ;resonance arg
        stor $resonance, t2
    .msgProcEnd:
    ret

.main:
    pop r2              ;r2 buffer size in frames
    pop r1              ;r1 buffer address
    lcons r3, $filters  ;filter array pointer
    load r4, $channels
    lcons t0, 0         ;iteration index
    lcons t4, 4         ;constant 4 bytes
    
    .processStart:
        jae t0, r4, .processEnd
        load_p r0, r3      ;filter handle
        
        int 0              ;enter
        push r4            ;arg 3 - frame skip
        push r2            ;arg 2 - frame count
        push r1            ;arg 1 - buffer address
        push r0            ;arg 0 - filter handle
        int 0x61           ;process biquad
        int 1              ;exit
        
        inc t0
        add r1, r1, t4
        add r3, r3, t4
        jmp .processStart
    .processEnd:

	lcons r0, 0        ;return success
	halt
